/*
 *
 * Network Client Library
 *
 * Copyright (c) 2012 Samsung Electronics Co., Ltd All Rights Reserved.
 *
 * PROPRIETARY/CONFIDENTIAL
 * This software is the confidential and proprietary information of
 * SAMSUNG ELECTRONICS ("Confidential Information").
 * You shall not disclose such Confidential Information and shall
 * use it only in accordance with the terms of the license agreement
 * you entered into with SAMSUNG ELECTRONICS.
 * SAMSUNG make no representations or warranties about the suitability
 * of the software, either express or implied, including but not limited
 * to the implied warranties of merchantability, fitness for a particular
 * purpose, or non-infringement. SAMSUNG shall not be liable for
 * any damages suffered by licensee as a result of using, modifying or
 * distributing this software or its derivatives.
 *
 */

 
#ifndef __NETWORK_PM_INTF_H__
#define __NETWORK_PM_INTF_H__


#ifdef __cplusplus
extern "C"
{
#endif /* __cplusplus */

/**
 * @file network-pm-intf.h
 * @brief This file defines the interface of Profile Manager with the application/Connection Manager.
*/

/**
 * \addtogroup  profile_managing
 * \{
*/

#ifndef DEPRECATED
#define DEPRECATED __attribute__((deprecated))
#endif
/*==================================================================================================
                                         INCLUDE FILES
==================================================================================================*/

#include "network-pm-wlan.h"

/*==================================================================================================
                                           CONSTANTS
==================================================================================================*/

/*==================================================================================================
                                             ENUMS
==================================================================================================*/

/**
 * @enum net_pdp_type_t
 * This enumeration defines the pdp protocol type.
 */
typedef enum
{
	/** Not defined */
	NET_PDP_TYPE_NONE	= 0x00,
	/** PDP-GPRS type */
	NET_PDP_TYPE_GPRS,
	/** PDP-EDGE type */
	NET_PDP_TYPE_EDGE,
	/** PDP-UMTS type */
	NET_PDP_TYPE_UMTS,
} net_pdp_type_t;

/**
 * @enum net_state_type_t
 * This enumeration defines the service state type.
 */
typedef enum
{
	/** Not defined */
	NET_STATE_TYPE_UNKNOWN	= 0x00,
	/** Idle state */
	NET_STATE_TYPE_IDLE,
	/** Failure state */
	NET_STATE_TYPE_FAILURE,
	/** Association state */
	NET_STATE_TYPE_ASSOCIATION,
	/** Configuration state */
	NET_STATE_TYPE_CONFIGURATION,
	/** Ready state */
	NET_STATE_TYPE_READY,
	/** Online state */
	NET_STATE_TYPE_ONLINE,
	/** Login state */
	NET_STATE_TYPE_DISCONNECT,
} net_state_type_t;


/*==================================================================================================
                                 STRUCTURES AND OTHER TYPEDEFS
==================================================================================================*/


/**
 * Profile data structures: Used between Application and PM Plug-in Interface
 */
typedef struct
{
	/** Specifies a protocol type */
	net_pdp_type_t  ProtocolType;
	/** Specifies a service type(Internet, MMS, WAP, etc...) */
	net_service_type_t ServiceType;
	/** Network Access Point Name */
	char            Apn[NET_PDP_APN_LEN_MAX+1];
	/** Authentication info of the PDP profile */
	net_auth_info_t	AuthInfo;
	/** Browser Home URL or MMS server URL */
	char            HomeURL[NET_HOME_URL_LEN_MAX+1];
	/** Sim Info Mcc */
	char Mcc[NET_SIM_INFO_LEN+1];
	/** Sim Info Mnc */
	char Mnc[NET_SIM_INFO_LEN+1];
	/** Indicates whether the use of static IP or not */
	char IsStatic;

	/** Indicates Roaming mode */
	char Roaming;
	/** This will be deprecated */
	char SetupRequired;

	/** network information */
	net_dev_info_t net_info;
} net_pdp_profile_info_t;

/**
 * Specific profile information related to each technology type
 */
typedef union
{
	/** PDP Profile Information */
	net_pdp_profile_info_t       Pdp;
	/** Wifi Profile Information */
	net_wifi_profile_info_t      Wlan;
} net_specific_profile_info_t;

/**
 * This is the profile structure exposed to applications.
 */
typedef struct
{
	/** Device Type of the profile */
	net_device_t  	profile_type;
	/** Profile name */
	char	ProfileName[NET_PROFILE_NAME_LEN_MAX+1];
	/** Specific profile information */
	net_specific_profile_info_t ProfileInfo;
	/** Service state */
	net_state_type_t        ProfileState;
	/** Favourite flag */
	char Favourite;
} net_profile_info_t;

/*
==================================================================================================
                                     FUNCTION PROTOTYPES
==================================================================================================
*/

/*****************************************************************************************/
/* net_add_profile API function prototype
 * int net_add_profile(net_service_type_t network_type, net_profile_info_t *prof_info);
 */

/**
 * \brief 	Add new Profile.
 *
 * \par Sync (or) Async:
 * This is a Synchronous API.
 *
 * \par Important Notes:
 *
 * \warning
 *  None
 *
 * \param[in]   network_type  A type of network service.
 * \param[in]   prof_info     A pointer of New created Profile Information to be added.
 *
 * \par Async Response Message:
 *        None.
 *
 * \return Return Type (int) \n
 * - NET_ERR_NONE  - indicating that the operation has completed successfully. \n
 * - NET_ERR_INVALID_PARAM - Invalid parameter\n
 * - NET_ERR_UNKNOWN - Any other error\n
 * - NET_ERR_APP_NOT_REGISTERED - Client is invalid may be unregistered\n
 *
 * \par Prospective Clients:
 * Network Connection Setting Applet, WLAN Setting UI Applet.
 *
 * \par Example of how this function would be called:
 *
 * net_profile_info_t prof_info;\n
 * int result; \n
 * result = net_add_profile( NET_SERVICE_MMS, &prof_info ); \n
 * if(result == NET_ERR_NONE)
 *
******************************************************************************************/
int net_add_profile(net_service_type_t network_type, net_profile_info_t *prof_info);

/*****************************************************************************************/
/* net_delete_profile API function prototype
 * int net_delete_profile(const char* profile_name);
 */

/**
 * \brief 	Delete a specific existing Profile.
 *              (cellular : Delete profile, wifi : forgot AP)
 *
 * \par Sync (or) Async:
 * This is a Synchronous API.
 *
 * \par Important Notes:
 *
 * \warning
 *  None
 *
 * \param[in]   profile_name        Profile name to be deleted.
 *
 * \par Async Response Message:
 *        None.
 *
 * \return Return Type (int) \n
 * - NET_ERR_NONE  - indicating that the operation has completed successfully.
 * - NET_ERR_INVALID_PARAM - Invalid parameter
 * - NET_ERR_UNKNOWN - Any other error
 * - NET_ERR_APP_NOT_REGISTERED - Client is invalid may be unregistered
 *
 * \par Prospective Clients:
 * Network Connection Setting Applet, WLAN Setting UI Applet.
 *
 * \par Example of how this function would be called:
 *
 * int result;\n
 *
 * result = net_delete_profile(profile_name);
 *
 * if (result == NET_ERR_NONE)
 *
******************************************************************************************/
int net_delete_profile(const char *profile_name);

/*****************************************************************************************/
/* net_get_profile_info API function prototype
 * int net_get_profile_info(const char *profile_name, net_profile_info_t *prof_info);
 */

/**
 * \brief 	Return the profile information referred by Profile Name.
 *
 * \par Sync (or) Async:
 * This is a Synchronous API.
 *
 * \par Important Notes:
 *		On success, profile information shall be copied to prof_info parameter in net_profile_info_t format.
 *		If profile doesn't exist, error shall be returned.
 *
 * \warning
 *  None
 *
 * \param[in]   profile_name  Profile Identifier.
 * \param[out]  prof_info     Buffer containing the profile.
 *
 * \par Async Response Message:
 *        None.
 *
 * \return Return Type (int) \n
 * - NET_ERR_NONE  - indicating that the operation has completed successfully. \n
 * - NET_ERR_INVALID_PARAM - Invalid parameter\n
 * - NET_ERR_UNKNOWN - Any other error\n
 * - NET_ERR_APP_NOT_REGISTERED - Client is invalid may be unregistered\n
 *
 * \par Prospective Clients:
 * Profile Manager.
 *
 * \par Example of how this function would be called:
 *
 * net_profile_info_t prof_info;\n
 * int result;\n
 *
 * result = net_get_profile_info(profile_name, &prof_info);\n
 *
 * if (result == NET_ERR_NONE)\n
 *
******************************************************************************************/
int net_get_profile_info(const char *profile_name, net_profile_info_t *prof_info);

/*****************************************************************************************/
/* net_modify_profile API function prototype
 * int net_modify_profile(const char* profile_name, net_profile_info_t* prof_info);
 */

/**
 * \brief 	Edit a specific existing Profile.
 *
 * \par Sync (or) Async:
 * This is a Synchronous API.
 *
 * \par Important Notes:
 *
 * \warning
 *  None
 *
 * \param[in]  profile_name  Profile Identifier.
 * \param[in]  prof_info     Changed Profile Information to be updated.
 *
 *
 * \par Async Response Message:
 *        None.
 *
 * \return Return Type (int) \n
 * - NET_ERR_NONE  - indicating that the operation has completed successfully. \n
 * - NET_ERR_INVALID_PARAM - Invalid parameter\n
 * - NET_ERR_UNKNOWN - Any other error\n
 * - NET_ERR_APP_NOT_REGISTERED - Client is invalid may be unregistered\n
 *
 * \par Prospective Clients:
 * Network Connection Setting Applet.
 *
 * \par Example of how this function would be called:
 *
 * net_profile_info_t prof_info;\n
 * int result;\n
 *
 * result = net_get_profile_info(profile_name, &prof_info);\n
 * ......(Modifying ProfInfo)\n
 *
 * result = net_modify_profile(profile_name, &prof_info);\n
 *
 * if (result == NET_ERR_NONE)\n
 *
******************************************************************************************/
int net_modify_profile(const char *profile_name, net_profile_info_t *prof_info);

/*****************************************************************************************/
/**
 * @fn   int net_get_profile_list(net_device_t device_type, net_profile_info_t **profile_list, int *count)
 *
 * This function request service list to ConnMan through dbus.
 *
 * \par Sync (or) Async:
 * This is a Synchronous API.
 *
 * \param[in]	device_type     Type of device
 * \param[out]	profile_list    Profile list. After use this, it should be free()
 * \param[out]	count           Number of profile
 *
 * \return Return Type (int) \n
 * - NET_ERR_NONE  - indicating that the operation has completed successfully. \n
 * - NET_ERR_INVALID_PARAM - Invalid parameter\n
 * - NET_ERR_UNKNOWN - Any other error\n
 * - NET_ERR_APP_NOT_REGISTERED - Client is invalid may be unregistered\n
 *
 * \par Prospective Clients:
 * External Apps.
 *
******************************************************************************************/
int net_get_profile_list(net_device_t device_type, net_profile_info_t **profile_list, int *count);

/**
 * \}
 */

#ifdef __cplusplus
}
#endif /* __cplusplus */

#endif


